@@ -45,7 +45,7 @@
 #include "gpm-ac-adapter.h"
 #include "gpm-button.h"
 #include "gpm-backlight.h"
-#include "gpm-brightness-lcd.h"
+#include "gpm-brightness.h"
 #include "gpm-conf.h"
 #include "gpm-control.h"
 #include "gpm-common.h"
@@ -65,7 +65,7 @@
 struct GpmBacklightPrivate
 {
 	GpmAcAdapter		*ac_adapter;
-	GpmBrightnessLcd	*brightness;
+	GpmBrightness		*brightness;
 	GpmButton		*button;
 	GpmConf			*conf;
 	GpmFeedback		*feedback;
@@ -81,6 +81,7 @@ struct GpmBacklightPrivate
 	GTimer			*idle_timer;
 	gfloat			 ambient_sensor_value;
 	guint			 idle_dim_timeout;
+	guint			 master_percentage;
 };
 
 enum {
@@ -89,7 +90,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint	     signals [LAST_SIGNAL] = { 0, };
+static guint	     signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GpmBacklight, gpm_backlight, G_TYPE_OBJECT)
 
@@ -136,7 +137,7 @@ gpm_backlight_sync_policy (GpmBacklight *backlight)
 
 	error = NULL;
 
-	if (on_ac == TRUE) {
+	if (on_ac) {
 		gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC, &timeout);
 		gpm_conf_get_string (backlight->priv->conf, GPM_CONF_BACKLIGHT_DPMS_METHOD_AC, &dpms_method);
 	} else {
@@ -157,7 +158,7 @@ gpm_backlight_sync_policy (GpmBacklight *backlight)
 	/* choose a sensible default */
 	if (method == GPM_DPMS_METHOD_DEFAULT) {
 		gpm_debug ("choosing sensible default");
-		if (backlight->priv->is_laptop == TRUE) {
+		if (backlight->priv->is_laptop) {
 			gpm_debug ("laptop, so use GPM_DPMS_METHOD_OFF");
 			method = GPM_DPMS_METHOD_OFF;
 		} else {
@@ -256,7 +257,7 @@ gpm_backlight_get_mode (GpmBacklight *backlight,
 	}
 
 	ret = gpm_dpms_get_mode_enum (backlight->priv->dpms, &mode, error);
-	if (ret == TRUE) {
+	if (ret) {
 		*mode_str = g_strdup (gpm_dpms_mode_to_string (mode));
 	}
 	return ret;
@@ -266,9 +267,7 @@ gpm_backlight_get_mode (GpmBacklight *backlight,
  * gpm_backlight_get_brightness:
  **/
 gboolean
-gpm_backlight_get_brightness (GpmBacklight *backlight,
-			      guint	   *brightness,
-			      GError	  **error)
+gpm_backlight_get_brightness (GpmBacklight *backlight, guint *brightness, GError **error)
 {
 	guint level;
 	gboolean ret;
@@ -285,8 +284,8 @@ gpm_backlight_get_brightness (GpmBacklight *backlight,
 	}
 
 	/* gets the current brightness */
-	ret = gpm_brightness_lcd_get (backlight->priv->brightness, &level);
-	if (ret == TRUE) {
+	ret = gpm_brightness_get (backlight->priv->brightness, &level);
+	if (ret) {
 		*brightness = level;
 	} else {
 		*error = g_error_new (gpm_backlight_error_quark (),
@@ -300,10 +299,11 @@ gpm_backlight_get_brightness (GpmBacklight *backlight,
  * gpm_backlight_set_brightness:
  **/
 gboolean
-gpm_backlight_set_brightness (GpmBacklight *backlight,
-			      guint	    brightness,
-			      GError	   **error)
+gpm_backlight_set_brightness (GpmBacklight *backlight, guint percentage, GError **error)
 {
+	gboolean ret;
+	gboolean hw_changed;
+
 	g_return_val_if_fail (backlight != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BACKLIGHT (backlight), FALSE);
 
@@ -315,20 +315,22 @@ gpm_backlight_set_brightness (GpmBacklight *backlight,
 		return FALSE;
 	}
 
-	/* just set the AC brightness for now, don't try to be clever */
-	gpm_conf_set_uint (backlight->priv->conf, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC, brightness);
-#if 0
-	gboolean ret;
+	/* just set the master percentage for now, don't try to be clever */
+	backlight->priv->master_percentage = percentage;
+
 	/* sets the current policy brightness */
-	ret = gpm_brightness_lcd_set_std (backlight->priv->brightness, brightness);
-	if (ret == FALSE) {
+	ret = gpm_brightness_set (backlight->priv->brightness, percentage, &hw_changed);
+	if (!ret) {
 		*error = g_error_new (gpm_backlight_error_quark (),
 				      GPM_BACKLIGHT_ERROR_GENERAL,
 				      "Cannot set policy brightness");
 	}
-#endif
-
-	return TRUE;
+	/* we emit a signal for the brightness applet */
+	if (ret && hw_changed) {
+		gpm_debug ("emitting brightness-changed : %i", percentage);
+		g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, percentage);
+	}
+	return ret;
 }
 
 /**
@@ -352,10 +354,12 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 {
 	gfloat brightness;
 	gfloat scale;
+	gboolean ret;
 	gboolean on_ac;
 	gboolean do_laptop_lcd;
 	gboolean enable_action;
 	gboolean battery_reduce;
+	gboolean hw_changed;
 	guint value;
 	guint old_value;
 
@@ -370,9 +374,8 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 		return FALSE;
 	}
 
-	/* get the 'main' brightness */
-	gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC, &value);
-	brightness = value / 100.0f;
+	/* get the last set brightness */
+	brightness = backlight->priv->master_percentage / 100.0f;
 	gpm_debug ("1. main brightness %f", brightness);
 
 	/* get AC status */
@@ -380,8 +383,12 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 
 	/* reduce if on battery power if we should */
 	gpm_conf_get_bool (backlight->priv->conf, GPM_CONF_BACKLIGHT_BATTERY_REDUCE, &battery_reduce);
-	if (on_ac == FALSE && battery_reduce == TRUE) {
-		gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_BACKLIGHT_BRIGHTNESS_BATT, &value);
+	if (on_ac == FALSE && battery_reduce) {
+		gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_BACKLIGHT_BRIGHTNESS_DIM_BATT, &value);
+		if (value > 100) {
+			gpm_warning ("cannot use battery brightness value %i, correcting to 50", value);
+			value = 50;
+		}
 		scale = (100 - value) / 100.0f;
 		brightness *= scale;
 	} else {
@@ -390,13 +397,17 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	gpm_debug ("2. battery scale %f, brightness %f", scale, brightness);
 
 	/* reduce if system is momentarily idle */
-	if (on_ac == TRUE) {
+	if (on_ac) {
 		gpm_conf_get_bool (backlight->priv->conf, GPM_CONF_BACKLIGHT_IDLE_DIM_AC, &enable_action);
 	} else {
 		gpm_conf_get_bool (backlight->priv->conf, GPM_CONF_BACKLIGHT_IDLE_DIM_BATT, &enable_action);
 	}
-	if (enable_action == TRUE && backlight->priv->system_is_idle == TRUE) {
+	if (enable_action && backlight->priv->system_is_idle == TRUE) {
 		gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_BACKLIGHT_IDLE_BRIGHTNESS, &value);
+		if (value > 100) {
+			gpm_warning ("cannot use idle brightness value %i, correcting to 50", value);
+			value = 50;
+		}
 		scale = value / 100.0f;
 		brightness *= scale;
 	} else {
@@ -406,7 +417,7 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 
 	/* reduce if ambient is low */
 	gpm_conf_get_bool (backlight->priv->conf, GPM_CONF_AMBIENT_ENABLE, &enable_action);
-	if (backlight->priv->can_sense == TRUE && enable_action == TRUE) {
+	if (backlight->priv->can_sense && enable_action == TRUE) {
 		gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_AMBIENT_SCALE, &value);
 		scale = backlight->priv->ambient_sensor_value * (value / 100.0f);
 		gpm_conf_get_uint (backlight->priv->conf, GPM_CONF_AMBIENT_FACTOR, &value);
@@ -425,25 +436,26 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	gpm_debug ("4. ambient scale %f, brightness %f", scale, brightness);
 
 	/* convert to percentage */
-	value = roundf (brightness * 100.0f);
+	value = (guint) ((brightness * 100.0f) + 0.5);
 
 	/* only do stuff if the brightness is different */
-	gpm_brightness_lcd_get (backlight->priv->brightness, &old_value);
+	gpm_brightness_get (backlight->priv->brightness, &old_value);
 	if (old_value == value) {
 		gpm_debug ("values are the same, no action");
 		return FALSE;
 	}
 
 	/* only show dialog if interactive */
-	if (interactive == TRUE) {
+	if (interactive) {
 		gpm_feedback_display_value (backlight->priv->feedback, (float) brightness);
 	}
 
+	ret = gpm_brightness_set (backlight->priv->brightness, value, &hw_changed);
 	/* we emit a signal for the brightness applet */
-	gpm_debug ("emitting brightness-changed : %i", value);
-	g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, value);
-
-	gpm_brightness_lcd_set_std (backlight->priv->brightness, value);
+	if (ret && hw_changed) {
+		gpm_debug ("emitting brightness-changed : %i", value);
+		g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, value);
+	}
 	return TRUE;
 }
 
@@ -453,17 +465,18 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
  * We might have to do things when the gconf keys change; do them here.
  **/
 static void
-conf_key_changed_cb (GpmConf      *conf,
-		     const gchar  *key,
-		     GpmBacklight *backlight)
+conf_key_changed_cb (GpmConf *conf, const gchar *key, GpmBacklight *backlight)
 {
 	gboolean on_ac;
 	on_ac = gpm_ac_adapter_is_present (backlight->priv->ac_adapter);
 
-	if (strcmp (key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
+	if (on_ac && strcmp (key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
+		gpm_conf_get_uint (backlight->priv->conf,
+				   GPM_CONF_BACKLIGHT_BRIGHTNESS_AC,
+				   &backlight->priv->master_percentage);
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 
-	} else if (strcmp (key, GPM_CONF_BACKLIGHT_BRIGHTNESS_BATT) == 0) {
+	} else if (!on_ac && strcmp (key, GPM_CONF_BACKLIGHT_BRIGHTNESS_DIM_BATT) == 0) {
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 	}
 
@@ -473,6 +486,7 @@ conf_key_changed_cb (GpmConf      *conf,
 	         strcmp (key, GPM_CONF_AMBIENT_SCALE) == 0 ||
 	         strcmp (key, GPM_CONF_BACKLIGHT_ENABLE) == 0 ||
 	         strcmp (key, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT) == 0 ||
+	         strcmp (key, GPM_CONF_BACKLIGHT_BATTERY_REDUCE) == 0 ||
 	         strcmp (key, GPM_CONF_BACKLIGHT_IDLE_BRIGHTNESS) == 0) {
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 
@@ -481,6 +495,12 @@ conf_key_changed_cb (GpmConf      *conf,
 	           strcmp (key, GPM_CONF_BACKLIGHT_DPMS_METHOD_AC) == 0 ||
 	           strcmp (key, GPM_CONF_BACKLIGHT_DPMS_METHOD_BATT) == 0) {
 		gpm_backlight_sync_policy (backlight);
+	} else if (strcmp (key, GPM_CONF_BACKLIGHT_IDLE_DIM_TIME) == 0) {
+		gpm_conf_get_uint (backlight->priv->conf,
+				   GPM_CONF_BACKLIGHT_IDLE_DIM_TIME,
+				   &backlight->priv->idle_dim_timeout);
+	} else {
+		gpm_debug ("unknown key %s", key);
 	}
 }
 
@@ -501,33 +521,53 @@ ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
 }
 
 /**
- * button_pressed_cb:
+ * gpm_backlight_button_pressed_cb:
  * @power: The power class instance
  * @type: The button type, e.g. "power"
  * @state: The state, where TRUE is depressed or closed
  * @brightness: This class instance
  **/
 static void
-button_pressed_cb (GpmButton    *button,
-		   const gchar  *type,
-		   GpmBacklight *backlight)
+gpm_backlight_button_pressed_cb (GpmButton *button, const gchar *type, GpmBacklight *backlight)
 {
+	guint percentage;
+	gboolean ret;
+	gboolean hw_changed;
 	gpm_debug ("Button press event type=%s", type);
 
 	if (strcmp (type, GPM_BUTTON_BRIGHT_UP) == 0) {
+		/* go up one step */
+		ret = gpm_brightness_up (backlight->priv->brightness, &hw_changed);
 
-		if (backlight->priv->can_dim == TRUE) {
-			gpm_brightness_lcd_up (backlight->priv->brightness);
+		/* show the new value */
+		if (ret) {
+			gpm_brightness_get (backlight->priv->brightness, &percentage);
+			gpm_feedback_display_value (backlight->priv->feedback, (float) percentage/100.0f);
+			/* save the new percentage */
+			backlight->priv->master_percentage = percentage;
+		}
+		/* we emit a signal for the brightness applet */
+		if (ret && hw_changed) {
+			gpm_debug ("emitting brightness-changed : %i", percentage);
+			g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, percentage);
 		}
-
 	} else if (strcmp (type, GPM_BUTTON_BRIGHT_DOWN) == 0) {
+		/* go up down step */
+		ret = gpm_brightness_down (backlight->priv->brightness, &hw_changed);
 
-		if (backlight->priv->can_dim == TRUE) {
-			gpm_brightness_lcd_down (backlight->priv->brightness);
+		/* show the new value */
+		if (ret) {
+			gpm_brightness_get (backlight->priv->brightness, &percentage);
+			gpm_feedback_display_value (backlight->priv->feedback, (float) percentage/100.0f);
+			/* save the new percentage */
+			backlight->priv->master_percentage = percentage;
+		}
+		/* we emit a signal for the brightness applet */
+		if (ret && hw_changed) {
+			gpm_debug ("emitting brightness-changed : %i", percentage);
+			g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, percentage);
 		}
-
 	} else if (strcmp (type, GPM_BUTTON_LID_OPEN) == 0) {
-
 		/* make sure we undim when we lift the lid */
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 		gpm_backlight_sync_policy (backlight);
@@ -608,14 +648,13 @@ idle_changed_cb (GpmIdle      *idle,
 	GError *error;
 
 	/* don't dim or undim the screen when the lid is closed */
-	if (gpm_button_is_lid_closed (backlight->priv->button) == TRUE) {
+	if (gpm_button_is_lid_closed (backlight->priv->button)) {
 		return;
 	}
 
 	if (mode == GPM_IDLE_MODE_NORMAL) {
-
 		/* deactivate display power management */
-		if (backlight->priv->can_dpms == TRUE) {
+		if (backlight->priv->can_dpms) {
 			error = NULL;
 			gpm_dpms_set_active (backlight->priv->dpms, FALSE, &error);
 			if (error) {
@@ -632,9 +671,8 @@ idle_changed_cb (GpmIdle      *idle,
 		gpm_backlight_sync_policy (backlight);
 
 	} else if (mode == GPM_IDLE_MODE_SESSION) {
-
 		/* activate display power management */
-		if (backlight->priv->can_dpms == TRUE) {
+		if (backlight->priv->can_dpms) {
 			error = NULL;
 			gpm_dpms_set_active (backlight->priv->dpms, TRUE, &error);
 			if (error) {
@@ -676,20 +714,22 @@ mode_changed_cb (GpmDpms      *dpms,
 
 /**
  * brightness_changed_cb:
- * @brightness: The GpmBrightnessLcd class instance
+ * @brightness: The GpmBrightness class instance
  * @percentage: The new percentage brightness
  * @brightness: This class instance
  *
  * This callback is called when the brightness value changes.
  **/
 static void
-brightness_changed_cb (GpmBrightnessLcd *brightness,
-		       guint             percentage,
-		       GpmBacklight     *backlight)
+brightness_changed_cb (GpmBrightness *brightness, guint percentage, GpmBacklight *backlight)
 {
+	/* display the widget when something else changed the backlight */
 	gpm_debug ("Need to display backlight feedback value %i", percentage);
 	gpm_feedback_display_value (backlight->priv->feedback, (float) percentage / 100.0f);
 
+	/* save the new percentage */
+	backlight->priv->master_percentage = percentage;
+
 	/* we emit a signal for the brightness applet */
 	gpm_debug ("emitting brightness-changed : %i", percentage);
 	g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, percentage);
@@ -697,7 +737,7 @@ brightness_changed_cb (GpmBrightnessLcd *brightness,
 
 /**
  * brightness_changed_cb:
- * @brightness: The GpmBrightnessLcd class instance
+ * @brightness: The GpmBrightness class instance
  * @percentage: The new percentage brightness
  * @brightness: This class instance
  *
@@ -827,8 +867,13 @@ gpm_backlight_init (GpmBacklight *backlight)
 	g_signal_connect (backlight->priv->light_sensor, "sensor-changed",
 			  G_CALLBACK (sensor_changed_cb), backlight);
 
+	/* watch for manual brightness changes (for the feedback widget) */
+	backlight->priv->brightness = gpm_brightness_new ();
+	g_signal_connect (backlight->priv->brightness, "brightness-changed",
+			  G_CALLBACK (brightness_changed_cb), backlight);
+
 	/* gets caps */
-	backlight->priv->can_dim = gpm_brightness_lcd_has_hw ();
+	backlight->priv->can_dim = gpm_brightness_has_hw (backlight->priv->brightness);
 	backlight->priv->can_dpms = gpm_dpms_has_hw ();
 	backlight->priv->can_sense = gpm_light_sensor_has_hw (backlight->priv->light_sensor);
 
@@ -839,13 +884,13 @@ gpm_backlight_init (GpmBacklight *backlight)
 
 	/* expose ui in prefs program */
 	prefs_server = gpm_prefs_server_new ();
-	if (backlight->priv->is_laptop == TRUE) {
+	if (backlight->priv->is_laptop) {
 		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_LID);
 	}
-	if (backlight->priv->can_dim == TRUE) {
+	if (backlight->priv->can_dim) {
 		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_BACKLIGHT);
 	}
-	if (backlight->priv->can_sense == TRUE) {
+	if (backlight->priv->can_sense) {
 		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_AMBIENT);
 	}
 	g_object_unref (prefs_server);
@@ -863,10 +908,16 @@ gpm_backlight_init (GpmBacklight *backlight)
 			   GPM_CONF_GNOME_SS_PM_DELAY,
 			   backlight->priv->idle_dim_timeout);
 
+	/* set the main brightness, this is designed to be updated if the user changes the
+	 * brightness so we can undim to the 'correct' value */
+	gpm_conf_get_uint (backlight->priv->conf,
+			   GPM_CONF_BACKLIGHT_BRIGHTNESS_AC,
+			   &backlight->priv->master_percentage);
+
 	/* watch for brightness up and down buttons and also check lid state */
 	backlight->priv->button = gpm_button_new ();
 	g_signal_connect (backlight->priv->button, "button-pressed",
-			  G_CALLBACK (button_pressed_cb), backlight);
+			  G_CALLBACK (gpm_backlight_button_pressed_cb), backlight);
 
 	/* we use ac_adapter for the ac-adapter-changed signal */
 	backlight->priv->ac_adapter = gpm_ac_adapter_new ();
@@ -882,19 +933,12 @@ gpm_backlight_init (GpmBacklight *backlight)
 			   GPM_CONF_GNOME_SS_PM_DELAY,
 			   backlight->priv->idle_dim_timeout);
 
-	if (backlight->priv->can_dim == TRUE) {
-		/* watch for manual brightness changes (for the feedback widget) */
-		backlight->priv->brightness = gpm_brightness_lcd_new ();
-		g_signal_connect (backlight->priv->brightness, "brightness-changed",
-				  G_CALLBACK (brightness_changed_cb), backlight);
+	/* use a visual widget */
+	backlight->priv->feedback = gpm_feedback_new ();
+	gpm_feedback_set_icon_name (backlight->priv->feedback,
+				    GPM_STOCK_BRIGHTNESS_LCD);
 
-		/* use a visual widget */
-		backlight->priv->feedback = gpm_feedback_new ();
-		gpm_feedback_set_icon_name (backlight->priv->feedback,
-					    GPM_STOCK_BRIGHTNESS_LCD);
-	}
-
-	if (backlight->priv->can_dpms == TRUE) {
+	if (backlight->priv->can_dpms) {
 		/* DPMS mode poll class */
 		backlight->priv->dpms = gpm_dpms_new ();
 		g_signal_connect (backlight->priv->dpms, "mode-changed",
