@@ -112,7 +112,7 @@ gpm_brightness_kbd_get_hw (GpmBrightnessKbd *brightness,
 		gpm_debug ("ERROR: %s", error->message);
 		g_error_free (error);
 	}
-	if (ret == FALSE) {
+	if (!ret) {
 		/* abort as the DBUS method failed */
 		gpm_warning ("GetBrightness failed!");
 		return FALSE;
@@ -165,7 +165,7 @@ gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness,
 		gpm_debug ("ERROR: %s", error->message);
 		g_error_free (error);
 	}
-	if (ret == FALSE) {
+	if (!ret) {
 		/* abort as the DBUS method failed */
 		gpm_warning ("SetBrightness failed!");
 		return FALSE;
@@ -296,7 +296,7 @@ gpm_brightness_kbd_set_dim (GpmBrightnessKbd *brightness,
 		brightness->priv->level_dim_hw = brightness->priv->level_std_hw;
 	}
 	/* if in this state, then update */
-	if (brightness->priv->is_dimmed == TRUE) {
+	if (brightness->priv->is_dimmed) {
 		gpm_brightness_kbd_dim_hw (brightness, brightness->priv->level_dim_hw);
 	}
 	return TRUE;
@@ -340,7 +340,7 @@ gpm_brightness_kbd_dim (GpmBrightnessKbd *brightness)
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
 	/* check to see if we are already dimmed */
-	if (brightness->priv->is_dimmed == TRUE) {
+	if (brightness->priv->is_dimmed) {
 		gpm_warning ("already dim'ed");
 		return FALSE;
 	}
@@ -466,22 +466,6 @@ gpm_brightness_kbd_down (GpmBrightnessKbd *brightness)
 	return TRUE;
 }
 
-/**keyboard_backlight
- * gpm_brightness_kbd_constructor:
- **/
-static GObject *
-gpm_brightness_kbd_constructor (GType		  type,
-			        guint		  n_construct_properties,
-			        GObjectConstructParam *construct_properties)
-{
-	GpmBrightnessKbd      *brightness;
-	GpmBrightnessKbdClass *klass;
-	klass = GPM_BRIGHTNESS_KBD_CLASS (g_type_class_peek (GPM_TYPE_BRIGHTNESS_KBD));
-	brightness = GPM_BRIGHTNESS_KBD (G_OBJECT_CLASS (gpm_brightness_kbd_parent_class)->constructor
-			      		     (type, n_construct_properties, construct_properties));
-	return G_OBJECT (brightness);
-}
-
 /**
  * gpm_brightness_kbd_finalize:
  **/
@@ -518,7 +502,6 @@ gpm_brightness_kbd_class_init (GpmBrightnessKbdClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	object_class->finalize	   = gpm_brightness_kbd_finalize;
-	object_class->constructor  = gpm_brightness_kbd_constructor;
 
 	signals [BRIGHTNESS_CHANGED] =
 		g_signal_new ("brightness-changed",
@@ -584,7 +567,7 @@ adjust_kbd_brightness_according_to_ambient_light (GpmBrightnessKbd *brightness,
 
 	if (state == STATE_FORCED_UNKNOWN) {
 		/* if this is the first time we're launched with ambient light data... */
-		if (ambient_light < 50 ) {
+		if (ambient_light < 50) {
 			gpm_brightness_kbd_set_std (brightness, 100);
 			state = STATE_FORCED_ON;
 		} else {
@@ -592,7 +575,7 @@ adjust_kbd_brightness_according_to_ambient_light (GpmBrightnessKbd *brightness,
 			state = STATE_FORCED_OFF;
 		}
 	} else {
-		if (ambient_light < 30 && state != STATE_FORCED_ON ) {
+		if (ambient_light < 30 && state != STATE_FORCED_ON) {
 			/* if it's dark.. and we haven't already turned light on...
 			 *   => turn it on.. full blast! */
 			gpm_brightness_kbd_set_std (brightness, 100);
