@@ -38,7 +38,7 @@ gpm_st_start (GpmSelfTest *test, const gchar *name, GpmSelfTestClass class)
 	if (class == CLASS_MANUAL && test->class == CLASS_AUTO) {
 		return FALSE;
 	}
-	if (test->started == TRUE) {
+	if (test->started) {
 		g_print ("Not ended test! Cannot start!\n");
 		exit (1);
 	}	
@@ -221,7 +221,6 @@ main (int argc, char **argv)
 
 	/* manual */
 	gpm_st_run_test (test, gpm_st_graph_widget);
-//	gpm_st_run_test (test, gpm_st_webcam);
 
 #if 0
 	gpm_st_run_test (test, gpm_st_proxy);
