@@ -203,18 +203,31 @@ static void
 gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 {
 	GpmCell *cell;
+	GtkWidget *dialog;
+	GtkWidget *image;
 	gchar *icon_name;
 	gchar *description;
 
-	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
 	cell = g_object_get_data (G_OBJECT (item), "cell");
 
 	/* get long description */
 	description = gpm_cell_get_description (cell);
 	icon_name = gpm_cell_get_icon (cell);
-	gpm_notify_display (icon->priv->notify, _("Device information"), description,
-			    GPM_NOTIFY_TIMEOUT_NEVER, icon_name,
-			    GPM_NOTIFY_URGENCY_LOW);
+
+	image = gtk_image_new ();
+	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "%s",
+					 _("Device information"));
+	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
+						    "%s", description);
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), icon_name);
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name, GTK_ICON_SIZE_DIALOG);
+	gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+	gtk_widget_show (image);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
 	g_free (description);
 	g_free (icon_name);
 }
@@ -364,28 +377,6 @@ gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
 	gpm_debug ("clear tray (icon = %p)", icon);
 }
 
-/**
- * gpm_tray_icon_constructor:
- *
- * Connects the UI to the tray icon instance
- **/
-static GObject *
-gpm_tray_icon_constructor (GType		  type,
-			   guint		  n_construct_properties,
-			   GObjectConstructParam *construct_properties)
-{
-	GpmTrayIcon      *tray;
-	GpmTrayIconClass *klass;
-
-	klass = GPM_TRAY_ICON_CLASS (g_type_class_peek (GPM_TYPE_TRAY_ICON));
-
-	tray = GPM_TRAY_ICON (G_OBJECT_CLASS (gpm_tray_icon_parent_class)->constructor
-			      (type, n_construct_properties,
-			       construct_properties));
-
-	return G_OBJECT (tray);
-}
-
 /**
  * gpm_tray_icon_class_init:
  **/
@@ -395,7 +386,6 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize	   = gpm_tray_icon_finalize;
-	object_class->constructor  = gpm_tray_icon_constructor;
 
 	g_type_class_add_private (klass, sizeof (GpmTrayIconPrivate));
 
@@ -524,7 +514,7 @@ gpm_tray_icon_add_device (GpmTrayIcon  *icon,
 
 		/* generate the label */
 		desc = gpm_cell_unit_get_kind_localised (unit, FALSE);
-		label = g_strdup_printf ("%s (%i%%)", desc, unit->percentage);
+		label = g_strdup_printf ("%s (%.1f%%)", desc, unit->percentage);
 		item = gtk_image_menu_item_new_with_label (label);
 		g_free (label);
 
@@ -577,13 +567,13 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 	}
 
 	/* only do the seporator if we have at least one device and can do an action */
-	if (dev_cnt != 0 && (icon->priv->show_suspend == TRUE || icon->priv->show_hibernate == TRUE)) {
+	if (dev_cnt != 0 && (icon->priv->show_suspend || icon->priv->show_hibernate == TRUE)) {
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
 	/* Suspend if available */
-	if (icon->priv->show_suspend == TRUE) {
+	if (icon->priv->show_suspend) {
 		item = gtk_image_menu_item_new_with_mnemonic (_("_Suspend"));
 		image = gtk_image_new_from_icon_name (GPM_STOCK_SUSPEND, GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -593,7 +583,7 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 	}
 
 	/* Hibernate if available */
-	if (icon->priv->show_hibernate == TRUE) {
+	if (icon->priv->show_hibernate) {
 		item = gtk_image_menu_item_new_with_mnemonic (_("Hi_bernate"));
 		image = gtk_image_new_from_icon_name (GPM_STOCK_HIBERNATE, GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
